window.$ = window.$ || jQuery;

const jitsiProFrontend = {
    init() {
        this.bindEvents();
    },
    bindEvents() {
        $('#attendee_login_form').submit(this.handleMeetingLogin);
        $('.error-message').remove();
    },
    handleMeetingLogin(event) {
        // Prevent the default form submission
        event.preventDefault();

        // Remove any existing error messages
        $('.error-message').remove();

        // Get form data
        var email = $('#attendee_login_form .meeting-email').val().trim();
        var password = $('#attendee_login_form .meeting-pass').val().trim();


        // Validate form data
        var isValid = true;

        if (email === '') {
            // Display error message for empty email
            $('#attendee_login_form .meeting-email').after('<div class="error-message">Email field cannot be empty</div>');
            isValid = false;
        }

        if (password === '') {
            // Display error message for empty password
            $('#attendee_login_form .meeting-pass').after('<div class="error-message">Password field cannot be empty</div>');
            isValid = false;
        }

        // If form data is not valid, stop execution
        if (!isValid) {
            return;
        }

        var is_vendor;

        if ($("#attendee_login_form").hasClass("vendor_attendee_login_form")) {
            is_vendor = 'yes';
        } else {
            is_vendor = 'no';
        }

        // If form data is valid, proceed with AJAX request
        var formData = {
            action: 'jitsi_login_to_meeting',
            nonce: $('#attendee_login_form .nonce').val(),
            post_id: $('#attendee_login_form .post_id').val(),
            meeting_email: email,
            meeting_password: password,
            is_vendor: is_vendor,
        };

        // AJAX request
        $.ajax({
            type: 'POST',
            url: jitsi_pro_frontend.ajax_url,
            data: formData,
            success: function(response) {
                console.log(response);
                // Handle success response
                if (response.data === 'email_not_found') {
                    $('#attendee_login_form').prepend('<div class="error-message top">Invalid email or password</div>');
                } else if (response.data === 'incorrect_password') {
                    // Display error message for login failure
                    $('#attendee_login_form').prepend('<div class="error-message top">Invalid password</div>');
                } else if (response.data === 'login_successful') {
                    location.reload();
                }
            },
            error: function(xhr, status, error) {
                // Handle error response
                console.error(xhr.responseText);
                // You can display an error message to the user
            }
        });
    },
    isEmail(email) {
        var regex = /^([a-zA-Z0-9_.+-])+\@(([a-zA-Z0-9-])+\.)+([a-zA-Z0-9]{2,4})+$/;
        return regex.test(email);
    }

};

$(document).ready(function () {
    jitsiProFrontend.init();
});
