document.addEventListener("DOMContentLoaded", function() {

	document.querySelectorAll('.jitsi-field-switch input').forEach(function(element){
		element.addEventListener('change', function(e) {
			if(e.currentTarget.checked){
				element.closest(".jitsi-field-switch").classList.add('active');
			} else {
				element.closest(".jitsi-field-switch").classList.remove('active');
			}
		});
	});

	var jitsi_img_frame = [];
	document.addEventListener("click", e => {
		var el = ".jitsi-uploader-button, .jitsi-uploader-button *";
        var resetEl = ".jitsi-uploader-reset, .jitsi-uploader-reset *";
        var copyEl = '.jitsi-copy-sc, .jitsi-copy-sc *';

		if (e.target.matches(el)) {
            var that = e.target;
            var fieldForValue = that.getAttribute('data-for');

            // If the frame already exists, re-open it.
            if (jitsi_img_frame[fieldForValue]) {
                jitsi_img_frame[fieldForValue].open();
                return;
            }

            // Sets up the media library frame
            jitsi_img_frame[fieldForValue] = wp.media.frames.jitsi_img_frame = wp.media({
                title: 'Select image',
                button: {text: 'Use this image'},
                library: {type: 'image'}
            });

            // Runs when an image is selected.
            jitsi_img_frame[fieldForValue].on('select', function () {
                var media_attachment = jitsi_img_frame[fieldForValue].state().get('selection').first().toJSON();
                document.getElementById(fieldForValue).value = media_attachment.url;
                if(document.getElementById(fieldForValue + '-prev') !== null){
                    document.getElementById(fieldForValue + '-prev').setAttribute('src', media_attachment.url);
                } else {
                    document.getElementById(fieldForValue).parentNode.parentNode.insertAdjacentHTML('afterbegin', `<div id="${fieldForValue}-prev-wrap" class="jitsi-uploader-preview-img"><img  id="${fieldForValue}-prev" src="${media_attachment.url}" alt="Preview Image Upload"/></div>`);
                }
            });
            // Opens the media library frame.
            jitsi_img_frame[fieldForValue].open();
        }

        if(e.target.matches(resetEl)) {
            var that = e.target;
            var fieldForValue = that.getAttribute('data-for');
            var elem = document.getElementById(fieldForValue + '-prev-wrap');

            document.getElementById(fieldForValue).value = '';
            elem.parentNode.removeChild(elem);
        }

        if(e.target.matches(copyEl)){
            e.preventDefault();
            var that = e.target;
            var toCopy = that.getAttribute('data-for');
            var copyText = document.getElementById(toCopy);
            copyText.select();
            copyText.setSelectionRange(0, 99999);
            document.execCommand("copy");
            document.getElementById('jitsi-shortcode-copied').classList.add('fade-in');
            setTimeout(() => {
                document.getElementById('jitsi-shortcode-copied').classList.remove('fade-in');
            }, 2000);
        }
	});
});

(function ($) {
    $(document).ready(function(){
        $('.jitsi_datepicker').each(function(){
            $(this).datetimepicker({
                format: 'Y-m-d H:i'
            });
        });

        $('.jitsi_timepicker').each(function(){
            $(this).datetimepicker({
                datepicker:false,
                format: 'H:i'
            });
        });

        $('[name^=jitsi_pro__]').on('change', function(){
            var field = $(this).attr('id');
            $('[data-depend]').each(function(){
                var that = this;
                var depends = $(this).data('depend');
                var depValues = $(this).data('value');
                if(depends.includes(field)){
                    var shouldDisplay = true;
                    $.each(depends, function(index, value){
                        if($('#' + value).is(':checkbox')){
                            var value = $('#' + value).is(':checked') ? 1 : 0;
                        } else {
                            var value = $('#' + value).val();
                        }

                        if(value != depValues[index]){
                            shouldDisplay = false;
                        }
                    });
                    if(shouldDisplay){
                        $(that).show();
                    } else {
                        $(that).hide();
                    }
                }
            });
        });

        function jitsi_manage_depend(el){
            let sourcedata = el.data('depend');
            let hide = false;
            $.each(sourcedata, function(index, data){
                if($(`[name=${data.field}]:checked`).val() != data.value){
                    hide = true;
                }
            });
            if(hide){
                el.closest('tr').css('display', 'none');
            } else {
                el.closest('tr').css('display', 'table-row');
            }
        }

        $('.jitsi-admin-field[data-depend]').each(function(){
            jitsi_manage_depend($(this));
        });

        $('body').on('change', '.jitsi-admin-field', function(){
            $('.jitsi-admin-field[data-depend]').each(function(){
                jitsi_manage_depend($(this));
            });
        });

        // Install Addon js
        if( jitsi_pro.is_ultimate_active ){
            $("#install_jitsi_multivendor").click(function(){
                $('#install_jitsi_multivendor .text').text("Installing");
                $('#install_jitsi_multivendor .install-jitsi-loader').show();
                $('.install').removeClass('install-hover');

                $(this).addClass("start-install");
    
                function activate_switcher() {   
                    $('.status').text("Activated");
                    $('.jitsi-field-switcher').addClass("active");
                    $('.switcher-round').addClass("checked");
                }
         
                let $post_data = {'action': 'install_multivendor_addon', 'nonce': jitsi_pro.nonce,'status': 'install' };
    
                $.ajax({
                    type: 'post',
                    dataType: 'json',
                    url: jitsi_pro.ajaxurl,
                    data: $post_data,
                    success: function (response) {
                        $(".not-installed").hide();
                        $(".installed").show();
    
                        if ( jitsi_pro.multivendor_active){
                            activate_switcher();
                        }
                    },
                    error: function (errorThrown) {
                  
                        let install_success = errorThrown.responseText.match("Plugin installed successfully.");

                        let install_faild = errorThrown.responseText.match("Installation package not available.");

                        if( install_success !== null ){                          
                            $(".not-installed").hide();
                            $(".installed").show();
                            $('.switcher-card .not-installed .status').removeClass("fail-color");
    
                            if ( jitsi_pro.multivendor_active){
                                activate_switcher();
                            }   
                        }

                        if( install_faild !== null ){   
                            $('.switcher-card .not-installed .status').text("Install failed"); 
                            $('.switcher-card .not-installed .status').addClass("fail-color");
                            $('.switcher-card .not-installed .install-jitsi-loader').hide(); 
                            $("#install_jitsi_multivendor").removeClass("start-install");
                            $("#install_jitsi_multivendor").addClass("install-hover");
                            $('#install_jitsi_multivendor .text').text("Install");
                        }

                           
                    },
                  });
    
    
              });
    
            //Addons js activate & deactivate
            $('input[name^="jitsi_multivendor_addon_switcher"]').click(function () {
                var status;
                
                //Enable adddons
                function enable_addon(){                  
                   // $('.switcher-card').removeClass("opacity");
                    $('.switcher-card .installed .status').text("Activated");
                    $('.jitsi-field-switcher').addClass("active");
                    $('.switcher-round').addClass("checked");
                    $('.switcher-card .installed .activate-jitsi-loader').hide();
                    $('#jitsi_multivendor_addon_switcher').removeAttr('disabled');
                    $('.jitsi-field-switcher').removeClass("disabled-switcher");
                    $('.switcher-card .installed .status').removeClass("update-color");
                    $('.switcher-card .installed .status').removeClass("fail-color"); 
               }
                //Disable adddons
                function disable_addon(){
                   //$('.switcher-card').addClass("opacity");
                   $(this).prop('disabled', false);
                   $('.switcher-card .installed .status').text("Deactivated");
                   $('.jitsi-field-switcher').removeClass("active");
                   $('.switcher-round').removeClass("checked");
                   $('.switcher-card .installed .activate-jitsi-loader').hide();
                   $('#jitsi_multivendor_addon_switcher').removeAttr('disabled');
                   $('.jitsi-field-switcher').removeClass("disabled-switcher");
                   $('.switcher-card .installed .status').removeClass("update-color");
                   $('.switcher-card .installed .status').removeClass("fail-color");
               }
    
                //Send status data thorough ajax
               function send_status_data(){
                $('.switcher-card .installed .activate-jitsi-loader').show();

                let $post_data = {'action': 'multivendor_addon', 'nonce': jitsi_pro.nonce,'status': status };
    
                 $.ajax({
                     type: 'post',
                     dataType: 'json',
                     url: jitsi_pro.ajaxurl,
                     data: $post_data,
                     success: function (response) {
                         if(response.status == 'diactivate' ){
                              disable_addon();
                         }

                         if(response.status == 'activate' ){
                             enable_addon();
                         }
    
                     },
                     error: function (errorThrown) {
                            enable_addon();
                     },
                   });
                   
               }
    
               //Check multivendor plugin activate or not
                if( jitsi_pro.multivendor_active ) {
    
                    if( ! $(this).parent().hasClass('active')  ) {
                        $(this).attr("checked","checked");
                        $('#jitsi_multivendor_addon_switcher').attr('disabled','disabled');
                        $('.jitsi-field-switcher').addClass("disabled-switcher");
                        status = "yes";
                        send_status_data();
                    }else{
                        $(".jitsi-multivendor-disable-modal").show();
                        $(".jitsi-multivendor-disable-modal .popup-btn-wrapper .proceed-btn").on("click", function () {
                           $(".jitsi-multivendor-disable-modal").hide();
                          $("#jitsi_multivendor_addon_switcher").removeAttr("checked");
                          $('#jitsi_multivendor_addon_switcher').attr('disabled','disabled');
                          $('.jitsi-field-switcher').addClass("disabled-switcher");
                           status = "no";
                           send_status_data();
                       });
                    }
    
                } else {
                    $(".jitsi-multivendor-modal").show();
                    disable_addon();
                }
    
            })
    
            $(".jitsi-multivendor-popup .close_btn").on("click", function () {
                $(".jitsi-multivendor-modal").hide();
              });
    
              $(".jitsi-multivendor-modal").on("click", function () {
                $(this).hide();
              });
    
              $('.jitsi-popup').on('click', function(e) {
                e.stopPropagation();
              });
    
            //Disable addon modal
            $(".jitsi-disable-multivendor-popup .close_btn").on("click", function () {
                $(".jitsi-multivendor-disable-modal").hide();
              });
    
              $('.jitsi-multivendor-disable-modal').on('click', function() {
                $(this).hide();
              });
    
              $('.jitsi-disable-popup').on('click', function(e) {
                e.stopPropagation();
              });
    
              $(".disable-addon-popup-wrapper .cancel-btn").on("click", function () {
                $(".jitsi-multivendor-disable-modal").hide();
              });

        } else {
            //Addons js when ultimate not activate
            $(".jitsi-add-ons-wrapper .free-switcher-card ").mouseenter(function(){
                var icon = '<span class="dashicons dashicons-lock"></span>';
                $(".install").text("Get Ultimate");
                $(".install").addClass("upgrade");
                $(".install").prepend(icon);
        
                //show pro offer popup for addon upgrade button
                $('.upgrade').on('click', function (e) {
                    e.preventDefault();
                    WPPOOL.Popup("webinar_and_video_conference_with_jitsi_meet").show();
                })
            });
        
            $(".jitsi-add-ons-wrapper .free-switcher-card ").mouseleave(function(){
                $(".install").removeClass("upgrade");
                $(".install").text("Install");
            });
        }

         // Update Addon js
         $(".jitsi-addon-update-available").click(function(){
            let $post_data = {'action': 'update_multivendor_addon', 'nonce': jitsi_pro.nonce };
            const self = this;

            $('.switcher-card .installed .activate-jitsi-loader').show();
            $('.switcher-card .installed .jitsi-addon-update-available').text("Updating");
            $('.switcher-card .installed .jitsi-addon-update-available').addClass("updating");
            $('.switcher-card .installed .jitsi-addon-update-available').removeClass("hover");
            $('.switcher-card .installed .jitsi-addon-update-available').prop('disabled', true);
    
            $.ajax({
                type: 'post',
                dataType: 'json',
                url: jitsi_pro.ajaxurl,
                data: $post_data,
                success: function (response) {
                },
                error: function (errorThrown) {

                    let failed_status = 'Could not install the new plugin.[]';
                    let update_success = errorThrown.responseText.match("Plugin updated successfully.");

                    if( errorThrown.responseText !== null && failed_status == errorThrown.responseText ){
                        $('.switcher-card .installed .activate-jitsi-loader').hide();
                        $('.switcher-card .installed .status').text("Update failed");
                        $('.switcher-card .installed .jitsi-addon-update-available').text("Update Available");
                        $('.switcher-card .installed .status').addClass("fail-color");
                        $('.switcher-card .installed .jitsi-addon-update-available').removeClass("updating");
                        $('.switcher-card .installed .jitsi-addon-update-available').prop('disabled', false);
                        $('.switcher-card .installed .jitsi-addon-update-available').addClass("hover");
                        $('.switcher-card .installed .status').removeClass("update-color");
                    }

                    if( update_success !== null ){                          
                        $(self).hide();
                        $('.switcher-card .installed .activate-jitsi-loader').hide();
                        $('.switcher-card .installed .status').text("Updated");  
                        $('.switcher-card .installed .status').addClass("update-color");
                        $('.switcher-card .installed .status').removeClass("fail-color");   
                    }
   
                },
              });

         });

        // Function to toggle the visibility of the custom post type name row
        function toggleCustomPostTypeName() {
            if ($('#jitsi_opt_update_post_type_slug').is(':checked')) {
                $('tr:has(#jitsi_opt_custom_post_type_slug)').show();
            } else {
                $('tr:has(#jitsi_opt_custom_post_type_slug)').hide();
            }
        }

        // Initial check when the document is ready
        toggleCustomPostTypeName();

        // Add change event listener to the checkbox
        $('#jitsi_opt_update_post_type_slug').change(function() {
            toggleCustomPostTypeName();
        });

    });
}(jQuery));